#include "SendFileThread.h"
#include "MySyncObject.h"		// CMySyncObjectを使うため
#include "SendRecvThread.h"		// CSendRecvThreadを使うため



CSendFileThread::CSendFileThread(CSendRecvThread *pCSendRecvThread, LPSTR pszFileName)
{
	// コンストラクタの引数を覚える
	m_pCSendRecvThread = pCSendRecvThread;
	m_pszFileName = (LPSTR)calloc(strlen(pszFileName) + 1, sizeof(char));
	DISABLE_C4996
	strcpy(m_pszFileName, pszFileName);
	ENABLE_C4996

	// 変数の初期化
	m_fIamZombie = FALSE;
	m_pCMySyncObject = new CMySyncObject();
	m_pCMySyncObject->Initialize();
}


CSendFileThread::~CSendFileThread()
{
	if(m_pCMySyncObject != NULL)
		m_pCMySyncObject->Uninitialize();
	SAFE_DELETE(m_pCMySyncObject)
	SAFE_FREE(m_pszFileName)
}


UINT CSendFileThread::DoWork()
{
	int		fdSrcFile;		// 送信ファイルディスクリプタ
	struct stat statInfo;	// ファイルの情報のための変数
	long	lFileLength;

	//----------------------------------------------------
	// ①ファイルをOpenして、送信開始要求パケットを送信する
	// ファイルのOpen
	DISABLE_C4996
	fdSrcFile = open(m_pszFileName, O_RDONLY);
	ENABLE_C4996
	if (fdSrcFile == -1)
		goto L_END;
	// ファイルの情報を取得(ファイルサイズ)
	if (fstat(fdSrcFile, &statInfo) != 0)
		goto L_END;
	lFileLength = statInfo.st_size;

	// ファイル名, ファイル長の送信
	if (SendSendFileStartPacket(m_pszFileName, lFileLength) == FALSE)
		goto L_END;
	//----------------------------------------------------
	// ★②ここで応答待ちをするようにする
	//----------------------------------------------------
	// ③④ファイルを読み込んでファイル送信中/ファイル送信完了パケットを送信する
	// ★この関数でエラーが起こったときは、中断コマンドを送るようにするのが良いでしょう
	if (SendSendFilePacket(fdSrcFile, lFileLength) == FALSE)
		goto L_END;
	//----------------------------------------------------
	// ★⑤ここで応答待ちをするようにする
	//----------------------------------------------------

L_END:
	// ファイルのclose
	if (fdSrcFile != -1)
		close(fdSrcFile);
	m_pCMySyncObject->Lock();
	m_fIamZombie = TRUE;		// ファイル送信が完了/失敗でゾンビ
	m_pCMySyncObject->UnLock();
	return(0);
}

//----------------------------------------------
// function
//		①ファイル送信開始コマンドの送信
// parameter
//		LPSTR	pszFileName	[in]ファイル名
//		int		iFileLength	[in]ファイル長
// retun
//		TRUE/FALSE
//----------------------------------------------
BOOL CSendFileThread::SendSendFileStartPacket(LPSTR pszFileName, int iFileLength)
{
	BOOL		fRet = FALSE;
	SendFileStartReqRec	*pSendFileStartReq = NULL;
	int			iPacketSize;
	int			iSize;

	// 共通ヘッダ部にデータを格納
	iPacketSize = sizeof(SendFileStartReqRec);
	pSendFileStartReq = (SendFileStartReqRec *)calloc(iPacketSize, sizeof(BYTE));
	memcpy(pSendFileStartReq->header.bMagicData, MAGIC_STRING, strlen(MAGIC_STRING));
	pSendFileStartReq->header.wCommand = htons(CMD_SEND_FILE_START_REQ);
	pSendFileStartReq->header.wDataLen = htons(sizeof(FileInfoRec));

	Locate(1, 8, 2);
	fprintf(stderr, "ファイル送信 %s %d", pszFileName, iFileLength);

	DISABLE_C4996
	sprintf(pSendFileStartReq->FileInfo.szFileName, "%s", pszFileName);
	sprintf(pSendFileStartReq->FileInfo.szFileSize, "%d", iFileLength);
	ENABLE_C4996

	// ファイル送信開始要求パケットの送信
	fRet = m_pCSendRecvThread->SetSendData((char *)pSendFileStartReq, iPacketSize);

	SAFE_FREE(pSendFileStartReq);
	return(fRet);
}

#define	READ_SIZE	(1024 * 10)			// ファイルの読み込み単位(ガバッと読みます)
//----------------------------------------------
// function
//		ファイルを読み込みながらファイル送信コマンドの送信
// parameter
//		int		fd			[in]ファイルディスクリプタ
//		int		iFileLength	[in]ファイル長
// retun
//		TRUE/FALSE
//----------------------------------------------
BOOL CSendFileThread::SendSendFilePacket(int fd, int iFileLength)
{
	BOOL			fRet = FALSE, fEnd = FALSE;
	LPBYTE			pbData = NULL;
	int				iReadSize, iTotalSize = 0;
	SendFileIngRec	*pSendFileIng = NULL;
	int				iPacketSize, iWaitCount = 0;

	pbData = (LPBYTE)calloc(READ_SIZE, sizeof(BYTE));
	while (!m_fStopFlag)
	{
		lseek(fd, iTotalSize, SEEK_SET);
		iReadSize = read(fd, pbData, READ_SIZE);
		iPacketSize = sizeof(HeaderRec) + sizeof(SendFileStatRec) + iReadSize;
		pSendFileIng = (SendFileIngRec *)calloc(iPacketSize, sizeof(BYTE));

		// 共通ヘッダ部にデータを格納する
		memcpy(pSendFileIng->header.bMagicData, MAGIC_STRING, strlen(MAGIC_STRING));
		pSendFileIng->header.wDataLen = htons((WORD)sizeof(SendFileStatRec) + iReadSize);
		// データ部にデータを格納する
		DISABLE_C4996
		sprintf(pSendFileIng->Stat.szOffset, "%d", iTotalSize);	// 読み込み位置
		sprintf(pSendFileIng->Stat.szSize, "%d", iReadSize);	// 読み込んだデータ数
		ENABLE_C4996
		if (iReadSize > 0)			// ファイルの途中を送信
		{
			Locate(1, 8, 1);
			fprintf(stderr, "ファイル送信中 %d %d %d\n", iTotalSize, iReadSize, iFileLength);
			// 共通ヘッダ部にデータを格納する
			pSendFileIng->header.wCommand = htons(CMD_SEND_FILE_ING);
			// データ部にデータを格納する
			memcpy(pSendFileIng->bData, pbData, iReadSize);
		}
		else						// ファイルの最後をまで読んだので完了を送信
		{
			Locate(1, 9, 1);
			fprintf(stderr, "ファイル送信完了 %d %d\n", iTotalSize, iReadSize);
			// 共通ヘッダ部にデータを格納する
			pSendFileIng->header.wCommand = htons(CMD_SEND_FILE_END_REQ);
			fEnd = TRUE;				// ファイル送信完了
		}
		// パケット全体を送信バッファに書きこむ
		fRet = m_pCSendRecvThread->SetSendData((char *)pSendFileIng, iPacketSize);
		SAFE_FREE(pSendFileIng)
		if (fRet == TRUE)				// パケットを書き込めた
		{
			if (fEnd == TRUE)			// 最後まで送信したので終わり
				break;
			iTotalSize += iReadSize;	// パケットを書き込めたので読み込みポインタ更新
		}
		else							// 書き込めなかったので読み込みポインタはそのまま
		{
			Locate(1, 10, 0);
			fprintf(stderr, "送信バッファの空き待ち(%d)\n", ++iWaitCount);	// 送信バッファに書けなかったのでちょっと待つ
			usleep(100 * 1000);							// ★READ_SIZEを送信するのに掛かる時間程度待つのが良いでしょう
		}
	}
	SAFE_FREE(pbData)
	return(fRet);
}


//----------------------------------------------
// function
//    このスレッドはゾンビ状態か
//	  別スレッドから参照される
// parameter
//    なし
// return
//	  TRUE/FALSE
//----------------------------------------------
BOOL CSendFileThread::IsZombie()
{
	BOOL	fRet;

	m_pCMySyncObject->Lock();
	fRet = m_fIamZombie;
	m_pCMySyncObject->UnLock();
	return(fRet);
}

